# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import, division
from .. import engine
import os


def shell(database='default'):
    """Runs the command-line client for the specified database.
    """

    target = engine[database]
    dialect = engine[database].dialect.name

    if dialect == 'mysql':
        args = ['mysql']

        if target.url.username:
            args += ['--user=%s' % target.url.username]

        if target.url.password:
            args += ['--password=%s' % target.url.password]

        if 'unix_socket' in target.url.query:
            args += ["--socket=%s" % target.url.query['unix_socket']]

        elif target.url.host:
            args += ["--host=%s" % target.url.host]

            if target.url.port:
                args += ["--port=%s" % target.url.port]

        if target.url.database:
            args += [target.url.database]

    elif dialect == 'sqlite':
        args = ['sqlite3', target.url.database]

    else:  # pragma: nocoverage
        raise RuntimeError(
            'Database shell not available for the dialect %r' % dialect)

    os.execvp(args[0], args)



# -*- coding: utf-8 -*-
import pytest
from alchemist import db
import flask
from wsgi_intercept import add_wsgi_intercept, remove_wsgi_intercept
from wsgi_intercept.requests_intercept import install, uninstall


#! Host at which the test server will bein intercepted at.
HOST = 'localhost'

#! Port at which the test server will being intercepted at.
PORT = 5000


@pytest.fixture(autouse=True, scope='session')
def fixture_server(request):
    # Install the WSGI interception layer.
    install()

    # Enable the WSGI interception layer.
    add_wsgi_intercept(HOST, PORT, lambda: flask.current_app)

    # Add a finalizer to remove the interception layer.
    def finalize():
        remove_wsgi_intercept(HOST, PORT)
        uninstall()

    request.addfinalizer(finalize)


@pytest.fixture(autouse=True, scope='session')
def fixture_database(request):
    # Clear out old database contents.
    db.clear(database=True)

    # Initialize the database access layer.
    db.init(database=True)

    # Ensure we clear out the database at the end.
    def finalizer():
        db.session.rollback()
        db.clear(database=True)

    request.addfinalizer(finalizer)


@pytest.fixture(autouse=True, scope='function')
def fixture_data(request):
    request.addfinalizer(lambda: db.flush())



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import, division
from .sql import init, clear, flush
from .shell import shell


__all__ = [
    'init', 'clear', 'flush',
    'shell',
]



# -*- coding: utf-8 -*-
from alchemist.test import settings
from flask import Flask
from os import path
import os
import sys
import alchemist


class TestSettings:

    def test_project(self):
        self.app = Flask('alchemist.tests.a')
        with settings(self.app):
            alchemist.configure(self.app)

            assert self.app.config.get('A_SETTING', 1)

    def test_missing_project(self):
        """
        Should succeed and not raise if the project does not contain
        a settings file.
        """

        self.app = Flask('alchemist')
        with settings(self.app):
            alchemist.configure(self.app)

    def test_component(self):
        self.app = Flask('alchemist.tests.a')
        with settings(self.app, COMPONENTS=['alchemist.tests.a.b']):
            alchemist.configure(self.app)

            assert self.app.config.get('B_SETTING', 1)
            assert self.app.config.get('A_SETTING', 5)

    def test_project_as_component(self):
        """
        Should not merge the project settings in twice even if
        it is listed in the COMPONENTS array.
        """

        self.app = Flask('alchemist.tests.a')
        components = ['alchemist.tests.a.b', 'alchemist.tests.a']
        with settings(self.app, COMPONENTS=components):
            alchemist.configure(self.app)

            assert self.app.config.get('B_SETTING', 1)
            assert self.app.config.get('A_SETTING', 5)

    def test_global_env(self):
        """
        Should override settings from the file specified in the
        ALCHEMIST_SETTINGS_MODULE environ variable.
        """

        filename = path.join(path.dirname(__file__), '../a/b/settings.py')
        os.environ['ALCHEMIST_SETTINGS_MODULE'] = filename

        self.app = Flask('alchemist.tests.a')
        with settings(self.app):
            alchemist.configure(self.app)

            assert self.app.config.get('B_SETTING', 1)
            assert self.app.config.get('A_SETTING', 5)

        del os.environ['ALCHEMIST_SETTINGS_MODULE']

    def test_project_env(self):
        """
        Should override settings from the file specified in the
        <project>_SETTINGS_MODULE environ variable.
        """

        filename = path.join(path.dirname(__file__), '../a/b/settings.py')
        os.environ['TESTS_A_SETTINGS_MODULE'] = filename

        self.app = Flask('alchemist.tests.a')
        with settings(self.app):
            alchemist.configure(self.app)

            assert self.app.config.get('B_SETTING', 1)
            assert self.app.config.get('A_SETTING', 5)

        del os.environ['TESTS_A_SETTINGS_MODULE']


class TestApplication:

    @staticmethod
    def _clear_cache():
        from alchemist import app
        type(app).__dict__['_find_application']._cache.clear()

    def setup(self):
        self._clear_cache()

    def teardown(self):
        self._clear_cache()

    def test_stack(self):
        from .a.example import application

        assert application.name == 'alchemist.tests.a'

    def test_env(self):
        os.environ['ALCHEMIST_APPLICATION'] = 'alchemist.tests.a'

        from .a.b.example import application

        assert application.name == 'alchemist.tests.a'

        del os.environ['ALCHEMIST_APPLICATION']

    def test_env_direct(self):
        os.environ['ALCHEMIST_APPLICATION'] = 'alchemist.tests.a.b'

        from alchemist.app import application

        assert application.name == 'alchemist.tests.a.b'

        del os.environ['ALCHEMIST_APPLICATION']


class TestTestingDetection:

    def setup(self):
        self.app = Flask('alchemist.tests.a')
        self.context = self.app.app_context()
        self.context.push()

    def teardown(self):
        self.context.pop()

    def test_nose(self):
        old = sys.argv
        sys.argv = ['nosetests']

        alchemist.configure(self.app)

        assert self.app.config['TESTING']

        sys.argv = old

    def test_py(self):
        old = sys.argv
        sys.argv = ['py.test']

        alchemist.configure(self.app)

        assert self.app.config['TESTING']

        sys.argv = old

    def test_unittest(self):
        old = sys.argv
        sys.argv = ['python', 'setup.py', 'test']

        alchemist.configure(self.app)

        assert self.app.config['TESTING']

        sys.argv = old

    def test_alchemist(self):
        old = sys.argv
        sys.argv = ['alchemist', 'test']

        alchemist.configure(self.app)

        assert self.app.config['TESTING']

        sys.argv = old

    def test_not(self):
        old = sys.argv
        sys.argv = ['alchemist', 'load', '../../test.py']

        alchemist.configure(self.app)

        assert not self.app.config['TESTING']

        sys.argv = old



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import, division
from alchemist import management
from flask import Flask


class TestManager:

    def setup(self):
        self.app = Flask('alchemist')
        self.app.config['COMPONENTS'] = ['alchemist']

    def test_discover_commands(self):
        """Should discover commands from registered components.
        """

        manager = management.Manager(self.app)

        assert 'run' in manager._commands



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import, division
from flask import Flask
import pytest
from alchemist.conf import settings


class TestSettings:

    def test_no_context(self):
        with pytest.raises(KeyError):
            settings['COMPONENTS']

        assert settings.get('COMPONENTS') is None
        assert 'COMPONENTS' not in settings
        assert len(settings) == 0
        assert len(list(iter(settings))) == 0

    def test_with_context(self):
        app = Flask('alchemist')
        app.config['A_SETTING'] = 1
        with app.app_context():
            assert 'A_SETTING' in settings
            assert settings['A_SETTING'] == 1
            assert len(settings) > 0
            assert len(list(iter(settings))) > 0



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import, division
from flask.ext import script, components
import colorama
import sys


class Manager(script.Manager):

    def __init__(self, app, **kwargs):

        # Initialize color output.
        if sys.stdout.isatty():  # pragma: nocoverage
            colorama.init()

        # Disable loading of default flask-script commands.
        kwargs.setdefault('with_default_commands', False)

        super(Manager, self).__init__(app, **kwargs)

        # Discover commands using the flask-components utility.
        for component in components.find('commands', app):
            for command in component.values():
                if (command and isinstance(command, type) and
                        issubclass(command, (script.Command, script.Manager))):
                    self.add_command(command)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import, division
from ._engine import engine
from ._session import Session, session
from .query import Query
from .model import (Model, _metadata as metadata, _registry_map as registry,
                    _metadata_map as component_metadata)
from .operations import (init, clear, flush, shell)


__all__ = [
    'engine',
    'Session',
    'session',
    'Query',
    'Model',
    'metadata',
    'component_metadata',
    'registry',
    'init',
    'clear',
    'flush',
    'shell',
]

# TODO: Support these options perhaps -- look into them at least.
# app.config.setdefault('SQLALCHEMY_NATIVE_UNICODE', None)
# app.config.setdefault('SQLALCHEMY_ECHO', False)
# app.config.setdefault('SQLALCHEMY_RECORD_QUERIES', None)
# app.config.setdefault('SQLALCHEMY_POOL_SIZE', None)
# app.config.setdefault('SQLALCHEMY_POOL_TIMEOUT', None)
# app.config.setdefault('SQLALCHEMY_POOL_RECYCLE', None)
# app.config.setdefault('SQLALCHEMY_MAX_OVERFLOW', None)
# app.config.setdefault('SQLALCHEMY_COMMIT_ON_TEARDOWN', False)
# app.config.setdefault('SQLALCHEMY_TRACK_MODIFICATIONS', True)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import, division
from flask.ext import script
from alchemist import db
from termcolor import colored
from collections import defaultdict
import sys


def _iter_context():
    for component, registry in db.registry.items():
        for name, model in registry.items():
            if not name.startswith('_'):
                yield name, model


def _make_context():
    """Create the namespace of items already pre-imported when using shell
    """
    namespace = {'db': db, 'session': db.session}
    namespace.update(_iter_context())
    return namespace


class Shell(script.Shell):
    """
    Runs a Python shell inside an application context with all
    models pre-loaded.
    """

    name = 'shell'

    @property
    def banner(self):
        text = colored('from alchemist import db\n', 'green')
        text += colored('session = db.session\n', 'green')

        modules = defaultdict(set)
        for name, model in _iter_context():
            modules[model.__module__].add(name)

        for module, name in modules.items():
            context = (module, ', '.join(name))
            text += colored('from %s import %s\n' % context, 'green')

        return text

    @banner.setter
    def banner(self, value):
        """Flask script attempts to set the banner to itself for some unknown
        reason.  So just don't bother."""
        pass

    def __init__(self, *args, **kwargs):
        # # Default the context maker.
        kwargs.setdefault('make_context', _make_context)
        kwargs.setdefault('banner')

        # Continue initialization.
        super(Shell, self).__init__(*args, **kwargs)

    def get_options(self):
        options = super(Shell, self).get_options()

        # Add an additional option that allows for piping of input
        # to the shell.
        options += script.Option('--pipe', action="store_true"),

        return options

    def run(self, pipe, *args, **kwargs):
        if pipe:
            # User is attempting to pipe in script through stdin.
            text = sys.stdin.read()
            exec(text, None, _make_context())
            return

        # Fallback to normal cycle.
        super(Shell, self).run(*args, **kwargs)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import, division
from flask.ext import script


class Server(script.Server):

    name = 'run'

    def __init__(self, *args, **kwargs):

        # Set all defaults to None as they will be defaulted in handle
        # when we have access to an application.
        kwargs.setdefault('port', None)
        kwargs.setdefault('host', None)
        kwargs.setdefault('use_debugger', None)
        kwargs.setdefault('use_reloader', None)
        kwargs.setdefault('threaded', None)
        kwargs.setdefault('processes', None)

        super(Server, self).__init__(*args, **kwargs)

    def handle(self, app, **kwargs):

        # Collect default server configuration from the application config.
        config = app.config
        kwargs = {k: v for k, v in kwargs.items() if v is not None}
        kwargs.setdefault('host', config.get('SERVER_HOST'))
        kwargs.setdefault('port', config.get('SERVER_PORT'))
        kwargs.setdefault('use_debugger', config.get('SERVER_DEBUG'))
        kwargs.setdefault('use_reloader', config.get('SERVER_RELOAD'))
        kwargs.setdefault('threaded', config.get('SERVER_THREADED'))
        kwargs.setdefault('processes', config.get('SERVER_PROCESSES'))

        super(Server, self).handle(app, **kwargs)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import, division
from alchemist.conf import defer

# Debugging.
DEBUG = True

# Default server configuration.
SERVER_HOST = '::1'
SERVER_PORT = 8000
SERVER_DEBUG = defer('DEBUG')
SERVER_RELOAD = defer('DEBUG')
SERVER_THREADED = False
SERVER_PROCESSES = 1

# List of registered components.
COMPONENTS = ['alchemist']

# Default metaclass to use for the declarative extension with SQLAlchemist.
MODEL_METACLASS = 'sqlalchemy.ext.declarative.api.DeclarativeMeta'



import sqlalchemy as sa
from alchemist import db


class Entity(db.Model):

    id = sa.Column(sa.Integer, primary_key=True, autoincrement=True)


class Box(db.Model):

    id = sa.Column(sa.Integer, primary_key=True, autoincrement=True)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import, division
from flask.ext.script import Command, Option
import pytest
from os import path
from alchemist import app
from alchemist.conf import settings
from importlib import import_module


class Test(Command):

    name = 'test'

    option_list = [
        Option(dest='names', nargs='*',
               help='The name of the component (or project) to test.'),
    ]

    capture_all_args = True

    def run(self, *args, **kwargs):

        collected = []
        names = kwargs.pop('names')
        default_names = {app.application.name} | set(settings['COMPONENTS'])
        for name in (names or default_names):

            # Get the tests module or package location.

            module = import_module(name)
            tests = path.join(path.dirname(module.__file__), 'tests')
            if not path.exists(tests):
                tests += '.py'
                if not path.exists(tests):
                    continue

            # Collect all test components that are not alchemist.

            if name != 'alchemist':
                collected.append(tests)

        # Run collected test modules.
        pytest.main(collected + list(*args))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from setuptools import setup, find_packages
from imp import load_source
import sys


test_requirements = [
    'pytest >= 2.4',
    'pytest-pep8 >= 1.0',
    'pytest-cov >= 1.6'
]

if sys.version_info[0] < 3:
    test_requirements += ['mock']
    mysql_requirements = ['oursql == 0.9.3']

else:
    mysql_requirements = ['oursql == 0.9.4']


setup(
    name='alchemist',
    version=load_source('', 'alchemist/_version.py').__version__,
    description='A server architecture built on top of a solid foundation '
                'provided by flask, sqlalchemy, and various extensions.',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Framework :: Flask',
        # 'Framework :: SQLAlchemy',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.3',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Utilities',
    ],
    author='Concordus Applications',
    author_email='support@concordusapps.com',
    url='http://github.com/concordusapps/alchemist',
    packages=find_packages('.'),
    entry_points={'pytest11': ['alchemist = alchemist.plugin']},
    dependency_links=[
        'https://launchpad.net/oursql/py3k/py3k-0.9.4/+download/'
        'oursql-0.9.4.tar.gz#egg=oursql-0.9.4',

        'git+git://github.com/kvesteri/sqlalchemy-utils.git'
        '#egg=sqlalchemy-utils-0.21.0',
    ],
    install_requires=[
        'colorama',
        'termcolor >= 1.1, < 1.2',
        'flask >= 0.10',
        'blinker >= 1.3',
        'flask-components >= 0.1',
        'flask-script >= 0.6.6',
        'sqlalchemy >= 0.8',
        'sqlalchemy-utils >= 0.21',
        'alembic >= 0.6, < 0.7',
        'pygments',
        'requests',
        'wsgi_intercept >= 0.6.0'
    ] + test_requirements,
    tests_require=test_requirements,
    extras_require={
        'mysql': mysql_requirements
    }
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import, division
from ._version import __version__, __version_info__  # noqa
from .app import configure


__all__ = [
    'configure',
]
